/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/* -*- c++ -*- */
#ifndef MPS_Address_H
#define MPS_Address_H

namespace MPS {

/**
 * Represents a resolved name for an object.
 */
class Address {
private:
  string transport;
  vector<string> params;

  /// Splits a resolved name string into its constituent parts.
  vector<string> splitResolvedName(string const &resolvedName);

  /// Sets this to be equivalent to the resolved name passed in.
  void become(string const &resolvedName);

  /// Scope the assignment operator as "private". It's deliberately unimplemented.
  Address const &operator=(Address const &other);

public:
  /// Initialise this from the resolved name string passed in.
  Address(string const &resolvedName) {
    become(resolvedName);
  }

  /// Copy constructor
  Address(Address const &other)
    : transport(other.transport),
      params(other.params)
  {}

  string getResolvedName() const;	///< Return the connection spec as a string
  string getTransport() const { return transport; }		///< Return the transport name

  /// Return the count of parameters in the connection spec
  int getParamCount() const { return params.size(); }

  /// Return a parameter string
  string getParam(unsigned int index) const {
    if (index >= params.size())
      return "";
    else
      return params[index];
  }

  /// Modify a parameter string
  void setParam(unsigned int index, string const &value) {
    if (index >= params.size())
      params.resize(index + 1);
    params[index] = value;
  }
};

}

#endif
