/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/* -*- c++ -*- */
#ifndef MPS_Exception_H
#define MPS_Exception_H

namespace MPS {

/**
 * Base class for most exceptions thrown by MPS. */
class MPSException {
private:
  string _message;	///< Message explaining the exception condition.
public:
  MPSException() {}
  MPSException(string const &message)
    : _message(message)
  {}

  string const &getMessage() const { return _message; }
};

/**
 * Superclass of all exceptions throwable by MPS interface
 * implementations. */
class UserException: public MPSException {
public:
  UserException()
    : MPSException("UserException thrown")
  {}
};

/**
 * Thrown when reading from a closed Connection or a too-short
 * InputStream. */
class MPSConnectionClosedException: public MPSException {
public:
  MPSConnectionClosedException() {}
  MPSConnectionClosedException(string const &when)
    : MPSException("Connection closed: " + when)
  {}
};

}

#endif
