/* -*- c++ -*- */
/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#ifndef MPS_Interface_H
#define MPS_Interface_H

namespace MPS {

/**
 * Base class which all MPS Interfaces (implementations and proxies)
 * must inherit from. */
class Interface: public Referenceable {
private:
  string resolvedName;

  friend class Transport;
  void _setResolvedName(string const &newName) {
    resolvedName = newName;
  }

public:
  Interface() {
    char buf[64];
    sprintf(buf, "mps:local:%lu", (long) this);
    resolvedName = buf;
  }

  virtual ~Interface() {}

  /**
   * Returns the resolved name of the object this interface
   * accesses/communicates with. <b>NOTE:</b> Defaults to returning
   * "this" in the "mps:local:" transport space! Proxies must override
   * this method, and servers (when bound) should call
   * _setResolvedName. */
  virtual string _getResolvedName() const {
    return resolvedName;
  }
};

}

#endif
