/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

/* -*- c++ -*- */
#ifndef MPS_Stream_H
#define MPS_Stream_H

namespace MPS {

/**
 * Used to stream data from a Connection.
 */
class InputStream {
protected:
  /**
   * Reads a single byte out of the message body.  Subclasses must
   * override this.
   *
   * @return the byte read */
  virtual int read() = 0;

public:
  virtual ~InputStream() {}

  // These methods are not const because they call read(), which modifies <i>state</i>.
  int readint();				///< Reads an encoded 32-bit signed int.
  string readstring();				///< Reads an encoded ASCIZ byte string.
  bool readbool();				///< Reads an encoded boolean.
  long long readlong();				///< Reads an encoded 64-bit signed long.
  char readchar();				///< Reads an encoded char.
  string readReference();			///< Reads a reference to another MPS object.
  float readfloat();				///< Reads an encoded 32-bit float.
};

/**
 * Used to stream data to a Connection.
 */
class OutputStream {
protected:
  string buffer;			///< The message's body

  /**
   * Appends a single byte to the message body.
   *
   * @param b the byte to append */
  void write(int b) {
    buffer.append(1, (char) b);
  }

public:
  OutputStream()
    : buffer()
  {}

  virtual ~OutputStream() {}

  int getLength() const { return buffer.size(); }		///< Returns the buffer length
  char const *getBody() const { return buffer.c_str(); }	///< Returns the buffer content

  void writeint(int what);			///< Serialises a 32-bit signed int.
  void writestring(string const &what);		///< Serialises an ASCIZ byte string.
  void writebool(bool what);			///< Serialises a boolean.
  void writelong(long long what);		///< Serialises a 64-bit signed long.
  void writechar(char what);			///< Serialises a char.
  void writeReference(ref<Interface> const &what);
						///< Serialises a reference to another MPS object.
  void writefloat(float what);			///< Serialises a 32-bit float.

  virtual void flush() {
    buffer.erase();
  }
};

}

#endif
