/* -*- c++ -*- */
/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#ifndef MPS_Transport_Fastmsg_H
#define MPS_Transport_Fastmsg_H

#include <mps/mps.h>
#include <fastmsg/Scheduler.h>

namespace MPS {

/**
 * Implements a Fastmsg-based Transport class for accessing and
 * serving MPS objects.
 *
 * Spawns a Scheduler Thread object for each object served by this
 * process across fastmsg.
 *
 * Since this uses fastmsg Scheduler Threads, you should call it
 * from SchedulerApp's appMain().
 */
class FastmsgTransport: public MPS::Transport {
private:
  static FastmsgTransport *instance;

  ref<NamingService> namingServer;
  Scheduler &sched;

  FastmsgTransport(string const &nsLoc, Scheduler &s);

protected:
  virtual ref<Connection> connectTo(Address const &connectionSpec);
  virtual string resolveObjectName(string const &objectName) const;
  virtual string registerServer(Server *server, Address const &spec);

public:
  static void initialise(string const &namingServerLocation,
			 Scheduler &s) {
    if (instance == 0) {
      instance = new FastmsgTransport(namingServerLocation, s);
    }
  }
};

}

#endif
