/* -*- c++ -*- */
/**************************************************************************
 Copyright (c) 2000-2001, Tony Garnock-Jones
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are
 met:

     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.

     * Redistributions in binary form must reproduce the above
       copyright notice, this list of conditions and the following
       disclaimer in the documentation and/or other materials provided
       with the distribution.

     * Neither the names of the copyright holders nor the names of this
       software's contributors may be used to endorse or promote
       products derived from this software without specific prior
       written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**************************************************************************/

#ifndef MPS_Transport_Simpl_H
#define MPS_Transport_Simpl_H

#include <mps/mps.h>
#include <map>

namespace MPS {

/**
 * Implements a Transport class for accessing and
 * serving MPS objects via the SIMPL protocol
 * (http://www.holoweb.net/~simpl/). */
class SimplTransport: public MPS::Transport {
private:
  static SimplTransport *instance;	///< The single, per-application instance of SimplTransport

  //---------------------------------------------------------------------------

  class SimplIOStream;		///< Subclass of MPS::InputStream and MPS::OutputStream
  class SimplConnection;	///< Subclass of MPS::Connection - holds FCID

  typedef map< int, Server * > serverMap_t;

  //---------------------------------------------------------------------------

  int nextOid;				///< Next OID available for registration here.

  string simplName;			///< Our local SIMPL server name
  string namingServerLocation;		///< Resolved name of the naming server to use
  ref<NamingService> namingServer;	///< Reference to the naming server object

  serverMap_t serverMap;		///< All OIDs we serve

  //---------------------------------------------------------------------------

  SimplTransport(string const &nsLoc,
		 string const &simplName);

protected:
  virtual ref<Connection> connectTo(Address const &connectionSpec);
  virtual string resolveObjectName(string const &objectName) const;
  virtual string registerServer(Server *server, Address const &spec);

  friend class SimplIOStream;
  static void pushMessage(void *sender, char const *msg, int msglen);
  static void *popMessage(char *msg, int &msglen);

public:
  /**
   * Creates the single per-application instance of SimplTransport, and
   * registers it using Transport::registerTransport().
   *
   * @param namingServerLocation resolved address of the naming server to use
   * @param simplName the name to bind to our SIMPL mailbox */
  static void initialise(string const &namingServerLocation,
			 string const &simplName) {
    if (instance == 0) {
      instance = new SimplTransport(namingServerLocation,
				    simplName);
    }
  }

  static void mainloop();
};

}

#endif
